#ifndef DSL_H
#define DSL_H

// Globale Variablen
unsigned char iDSL_Array_in[39];
unsigned char iDSL_Array_out[39];
unsigned char iDSL_Data[7];
unsigned char iLast_Byte = 0;
unsigned char iData_Count = 0;
unsigned char iChecksum = 0;
unsigned char iPacket_Lenght = 0;
unsigned char packet_get = 0;

// Funktions definition
void DSL_init(void);
void DSL_data_incoming(unsigned char);
void DSL_packet_incoming(void);

// Funktions Source
unsigned char DSL_get(unsigned char i) {
  if (i < 39) {
    return iDSL_Array_in[i];
  } else {
    return 0;
  }
}

float DSL_to_Angle(unsigned char cByte1, unsigned char cByte2) {
  float fAngle = 0.0;
  fAngle = 255 * cByte1 + cByte2;	// Bytes zusammenfgen
  fAngle = (fAngle - 11059) / 81.9156;	// Mitte zur NULL verschieben und auf Winkel skalieren
  return fAngle;
}

void DSL_init(void) {
  iDSL_Array_in[0] = 0xFF;             // Syncronisation
  iDSL_Array_in[1] = 0xFF;             // Syncronisation
  iDSL_Array_in[2] = 0x1F;             // Status Header
  iDSL_Array_in[9] = 0x10;             // 10,11,12,13 = Servo 0 + Servo 1; 14 = Checksumme
  iDSL_Array_in[15] = 0x12;            // 16,17,18,19 = Servo 2 + Servo 3; 20 = Checksumme
  iDSL_Array_in[21] = 0x14;
  iDSL_Array_in[27] = 0x16;
  iDSL_Array_in[33] = 0x18;
  iDSL_Array_out[0] = 0xFF;		// Syncronisation
  iDSL_Array_out[1] = 0xFF;		// Syncronisation
  iDSL_Array_out[2] = 0x1F;		// Status Header
  iDSL_Array_out[3] = 35;		// Band
  iDSL_Array_out[4] = 10;		// Channel
  iDSL_Array_out[5] = 254;		// RSSI
  iDSL_Array_out[6] = 254;		// Batterie
  iDSL_Array_out[7] = 0x01;		// Mode / Div. RSSI
  iDSL_Array_out[8] = 182;		// Checksumme 181?
  iDSL_Array_out[9] = 0x10;		// 10,11,12,13 = Servo 0 + Servo 1; 14 = Checksumme
  iDSL_Array_out[15] = 0x12; 		// 16,17,18,19 = Servo 2 + Servo 3; 20 = Checksumme
  iDSL_Array_out[21] = 0x14;
  iDSL_Array_out[27] = 0x16;
  iDSL_Array_out[33] = 0x18;
}

void DSL_packet_incoming(void) {
//  unsigned char i = 0;
  if ((iDSL_Data[0] & 0xF0) == 0x10) {
    switch (iDSL_Data[0]) {
      case 0x1F:  iDSL_Array_in[3] = iDSL_Data[1]; iDSL_Array_in[4] = iDSL_Data[2]; iDSL_Array_in[5] = iDSL_Data[3]; iDSL_Array_in[6] = iDSL_Data[4]; iDSL_Array_in[7] = iDSL_Data[5]; packet_get = 1; break;
      case 0x10:  iDSL_Array_in[10] = iDSL_Data[1]; iDSL_Array_in[11] = iDSL_Data[2]; iDSL_Array_in[12] = iDSL_Data[3]; iDSL_Array_in[13] = iDSL_Data[4]; break;
      case 0x12:  iDSL_Array_in[16] = iDSL_Data[1]; iDSL_Array_in[17] = iDSL_Data[2]; iDSL_Array_in[18] = iDSL_Data[3]; iDSL_Array_in[19] = iDSL_Data[4]; break;
      case 0x14:  iDSL_Array_in[22] = iDSL_Data[1]; iDSL_Array_in[23] = iDSL_Data[2]; iDSL_Array_in[24] = iDSL_Data[3]; iDSL_Array_in[25] = iDSL_Data[4]; break;
      case 0x16:  iDSL_Array_in[28] = iDSL_Data[1]; iDSL_Array_in[29] = iDSL_Data[2]; iDSL_Array_in[30] = iDSL_Data[3]; iDSL_Array_in[31] = iDSL_Data[4]; break;
      case 0x18:  iDSL_Array_in[34] = iDSL_Data[1]; iDSL_Array_in[35] = iDSL_Data[2]; iDSL_Array_in[36] = iDSL_Data[3]; iDSL_Array_in[37] = iDSL_Data[4]; break;
      default: break;
    }
  }
}

void DSL_data_incoming(unsigned char iByte) {
  if ((iByte == 0xFF) && (iLast_Byte == 0xFF)) {
    iData_Count = 0;
    iChecksum = 0;
    return;
  }
  
  if (iData_Count == 0) {
    if(iByte == 0x1F) {		// Status Packet
      iPacket_Lenght = 5;
    } else {			// Servo Packet
      iPacket_Lenght = 4;	
    }
  }
  
  if(iData_Count > iPacket_Lenght) {
  
    // Checksumme berechnen
    iChecksum = ~iChecksum;
    if (iChecksum == 0xFF) {
      iChecksum = 0xFE;
    }
    
    // Ckecksumme ok? -> Packet incoming
    if (iByte == iChecksum) {
      DSL_packet_incoming();
    }
    
    // Fr neue Daten vorbereiten
    iData_Count = 0;
    iChecksum = 0;
  
  } else {
    iDSL_Data[iData_Count] = iByte;
    iChecksum += iByte;
    iData_Count++;
  }
  
  // Letztes Byte fr Synconisation merken
  iLast_Byte = iByte;
}

#endif